#  Drakkar-Software OctoBot-Trading
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.


MOCKED_EXCHANGE_INFO = {
    "bybit": [{'id': 'BLZUSDT', 'symbol': 'BLZ/USDT:USDT', 'base': 'BLZ', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'BLZ', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 1.0, 'price': 1e-05}, 'limits': {'leverage': {'min': 1.0, 'max': 25.0}, 'amount': {'min': 1.0, 'max': 98500.0}, 'price': {'min': 1e-05, 'max': 19.99998}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'BLZUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'BLZ', 'quoteCoin': 'USDT', 'launchTime': '1654497673000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '5', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '25.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.00001', 'maxPrice': '19.99998', 'tickSize': '0.00001'}, 'lotSizeFilter': {'maxOrderQty': '98500', 'minOrderQty': '1', 'qtyStep': '1', 'postOnlyMaxOrderQty': '985000'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'BNBPERP', 'symbol': 'BNB/USD:USDC', 'base': 'BNB', 'quote': 'USD', 'settle': 'USDC', 'baseId': 'BNB', 'quoteId': 'USD', 'settleId': 'USDC', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.01, 'price': 0.05}, 'limits': {'leverage': {'min': 1.0, 'max': 50.0}, 'amount': {'min': 0.01, 'max': 250.0}, 'price': {'min': 0.05, 'max': 99999.9}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'BNBPERP', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'BNB', 'quoteCoin': 'USD', 'launchTime': '1664459242000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '2', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '50.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.05', 'maxPrice': '99999.90', 'tickSize': '0.05'}, 'lotSizeFilter': {'maxOrderQty': '250.00', 'minOrderQty': '0.01', 'qtyStep': '0.01', 'postOnlyMaxOrderQty': '1250.00'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDC', 'copyTrading': 'none'}}, {'id': 'BNBUSDT', 'symbol': 'BNB/USDT:USDT', 'base': 'BNB', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'BNB', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.01, 'price': 0.05}, 'limits': {'leverage': {'min': 1.0, 'max': 50.0}, 'amount': {'min': 0.01, 'max': 1300.0}, 'price': {'min': 0.05, 'max': 99999.9}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'BNBUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'BNB', 'quoteCoin': 'USDT', 'launchTime': '0', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '2', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '50.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.05', 'maxPrice': '99999.90', 'tickSize': '0.05'}, 'lotSizeFilter': {'maxOrderQty': '1300.00', 'minOrderQty': '0.01', 'qtyStep': '0.01', 'postOnlyMaxOrderQty': '13000.00'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'normalOnly'}}, {'id': 'BNXUSDT', 'symbol': 'BNX/USDT:USDT', 'base': 'BNX', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'BNX', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.1, 'price': 0.0005}, 'limits': {'leverage': {'min': 1.0, 'max': 25.0}, 'amount': {'min': 0.1, 'max': 35000.0}, 'price': {'min': 0.0005, 'max': 999.999}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'BNXUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'BNX', 'quoteCoin': 'USDT', 'launchTime': '1677564318000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '4', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '25.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.0005', 'maxPrice': '999.9990', 'tickSize': '0.0005'}, 'lotSizeFilter': {'maxOrderQty': '35000.0', 'minOrderQty': '0.1', 'qtyStep': '0.1', 'postOnlyMaxOrderQty': '175000.0'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'BOBAUSDT', 'symbol': 'BOBA/USDT:USDT', 'base': 'BOBA', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'BOBA', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.1, 'price': 5e-05}, 'limits': {'leverage': {'min': 1.0, 'max': 12.5}, 'amount': {'min': 0.1, 'max': 9900.0}, 'price': {'min': 5e-05, 'max': 99.9999}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'BOBAUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'BOBA', 'quoteCoin': 'USDT', 'launchTime': '1649231761000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '5', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '12.50', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.00005', 'maxPrice': '99.99990', 'tickSize': '0.00005'}, 'lotSizeFilter': {'maxOrderQty': '9900.0', 'minOrderQty': '0.1', 'qtyStep': '0.1', 'postOnlyMaxOrderQty': '99000.0'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'BSVUSDT', 'symbol': 'BSV/USDT:USDT', 'base': 'BSV', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'BSV', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.01, 'price': 0.05}, 'limits': {'leverage': {'min': 1.0, 'max': 25.0}, 'amount': {'min': 0.01, 'max': 180.0}, 'price': {'min': 0.05, 'max': 99999.9}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'BSVUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'BSV', 'quoteCoin': 'USDT', 'launchTime': '1639105708000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '2', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '25.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.05', 'maxPrice': '99999.90', 'tickSize': '0.05'}, 'lotSizeFilter': {'maxOrderQty': '180.00', 'minOrderQty': '0.01', 'qtyStep': '0.01', 'postOnlyMaxOrderQty': '1800.00'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'BSWUSDT', 'symbol': 'BSW/USDT:USDT', 'base': 'BSW', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'BSW', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 1.0, 'price': 0.0001}, 'limits': {'leverage': {'min': 1.0, 'max': 25.0}, 'amount': {'min': 1.0, 'max': 19500.0}, 'price': {'min': 0.0001, 'max': 199.9998}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'BSWUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'BSW', 'quoteCoin': 'USDT', 'launchTime': '1648017126000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '4', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '25.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.0001', 'maxPrice': '199.9998', 'tickSize': '0.0001'}, 'lotSizeFilter': {'maxOrderQty': '19500', 'minOrderQty': '1', 'qtyStep': '1', 'postOnlyMaxOrderQty': '195000'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'BTC-04AUG23', 'symbol': 'BTC/USD:USDC-230804', 'base': 'BTC', 'quote': 'USD', 'settle': 'USDC', 'baseId': 'BTC', 'quoteId': 'USD', 'settleId': 'USDC', 'type': 'future', 'spot': False, 'margin': None, 'swap': False, 'future': True, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': 1691136000000, 'expiryDatetime': '2023-08-04T08:00:00.000Z', 'strike': None, 'optionType': None, 'precision': {'amount': 0.001, 'price': 0.5}, 'limits': {'leverage': {'min': 1.0, 'max': 50.0}, 'amount': {'min': 0.001, 'max': 10.0}, 'price': {'min': 0.5, 'max': 999999.0}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'BTC-04AUG23', 'contractType': 'LinearFutures', 'status': 'Trading', 'baseCoin': 'BTC', 'quoteCoin': 'USD', 'launchTime': '1689321600000', 'deliveryTime': '1691136000000', 'deliveryFeeRate': '0', 'priceScale': '2', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '50.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.50', 'maxPrice': '999999.00', 'tickSize': '0.50'}, 'lotSizeFilter': {'maxOrderQty': '10.000', 'minOrderQty': '0.001', 'qtyStep': '0.001', 'postOnlyMaxOrderQty': '50.000'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDC', 'copyTrading': 'none'}}, {'id': 'BTC-11AUG23', 'symbol': 'BTC/USD:USDC-230811', 'base': 'BTC', 'quote': 'USD', 'settle': 'USDC', 'baseId': 'BTC', 'quoteId': 'USD', 'settleId': 'USDC', 'type': 'future', 'spot': False, 'margin': None, 'swap': False, 'future': True, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': 1691740800000, 'expiryDatetime': '2023-08-11T08:00:00.000Z', 'strike': None, 'optionType': None, 'precision': {'amount': 0.001, 'price': 0.5}, 'limits': {'leverage': {'min': 1.0, 'max': 50.0}, 'amount': {'min': 0.001, 'max': 10.0}, 'price': {'min': 0.5, 'max': 999999.0}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'BTC-11AUG23', 'contractType': 'LinearFutures', 'status': 'Trading', 'baseCoin': 'BTC', 'quoteCoin': 'USD', 'launchTime': '1689926400000', 'deliveryTime': '1691740800000', 'deliveryFeeRate': '0', 'priceScale': '2', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '50.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.50', 'maxPrice': '999999.00', 'tickSize': '0.50'}, 'lotSizeFilter': {'maxOrderQty': '10.000', 'minOrderQty': '0.001', 'qtyStep': '0.001', 'postOnlyMaxOrderQty': '50.000'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDC', 'copyTrading': 'none'}}, {'id': 'BTC-25AUG23', 'symbol': 'BTC/USD:USDC-230825', 'base': 'BTC', 'quote': 'USD', 'settle': 'USDC', 'baseId': 'BTC', 'quoteId': 'USD', 'settleId': 'USDC', 'type': 'future', 'spot': False, 'margin': None, 'swap': False, 'future': True, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': 1692950400000, 'expiryDatetime': '2023-08-25T08:00:00.000Z', 'strike': None, 'optionType': None, 'precision': {'amount': 0.001, 'price': 0.5}, 'limits': {'leverage': {'min': 1.0, 'max': 50.0}, 'amount': {'min': 0.001, 'max': 10.0}, 'price': {'min': 0.5, 'max': 999999.0}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'BTC-25AUG23', 'contractType': 'LinearFutures', 'status': 'Trading', 'baseCoin': 'BTC', 'quoteCoin': 'USD', 'launchTime': '1686297600000', 'deliveryTime': '1692950400000', 'deliveryFeeRate': '0', 'priceScale': '2', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '50.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.50', 'maxPrice': '999999.00', 'tickSize': '0.50'}, 'lotSizeFilter': {'maxOrderQty': '10.000', 'minOrderQty': '0.001', 'qtyStep': '0.001', 'postOnlyMaxOrderQty': '50.000'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDC', 'copyTrading': 'none'}}, {'id': 'BTC-28JUL23', 'symbol': 'BTC/USD:USDC-230728', 'base': 'BTC', 'quote': 'USD', 'settle': 'USDC', 'baseId': 'BTC', 'quoteId': 'USD', 'settleId': 'USDC', 'type': 'future', 'spot': False, 'margin': None, 'swap': False, 'future': True, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': 1690531200000, 'expiryDatetime': '2023-07-28T08:00:00.000Z', 'strike': None, 'optionType': None, 'precision': {'amount': 0.001, 'price': 0.5}, 'limits': {'leverage': {'min': 1.0, 'max': 50.0}, 'amount': {'min': 0.001, 'max': 10.0}, 'price': {'min': 0.5, 'max': 999999.0}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'BTC-28JUL23', 'contractType': 'LinearFutures', 'status': 'Trading', 'baseCoin': 'BTC', 'quoteCoin': 'USD', 'launchTime': '1683273600000', 'deliveryTime': '1690531200000', 'deliveryFeeRate': '0', 'priceScale': '2', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '50.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.50', 'maxPrice': '999999.00', 'tickSize': '0.50'}, 'lotSizeFilter': {'maxOrderQty': '10.000', 'minOrderQty': '0.001', 'qtyStep': '0.001', 'postOnlyMaxOrderQty': '50.000'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDC', 'copyTrading': 'none'}}, {'id': 'BTC-29DEC23', 'symbol': 'BTC/USD:USDC-231229', 'base': 'BTC', 'quote': 'USD', 'settle': 'USDC', 'baseId': 'BTC', 'quoteId': 'USD', 'settleId': 'USDC', 'type': 'future', 'spot': False, 'margin': None, 'swap': False, 'future': True, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': 1703836800000, 'expiryDatetime': '2023-12-29T08:00:00.000Z', 'strike': None, 'optionType': None, 'precision': {'amount': 0.001, 'price': 0.5}, 'limits': {'leverage': {'min': 1.0, 'max': 50.0}, 'amount': {'min': 0.001, 'max': 10.0}, 'price': {'min': 0.5, 'max': 999999.0}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'BTC-29DEC23', 'contractType': 'LinearFutures', 'status': 'Trading', 'baseCoin': 'BTC', 'quoteCoin': 'USD', 'launchTime': '1680854400000', 'deliveryTime': '1703836800000', 'deliveryFeeRate': '0', 'priceScale': '2', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '50.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.50', 'maxPrice': '999999.00', 'tickSize': '0.50'}, 'lotSizeFilter': {'maxOrderQty': '10.000', 'minOrderQty': '0.001', 'qtyStep': '0.001', 'postOnlyMaxOrderQty': '50.000'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDC', 'copyTrading': 'none'}}, {'id': 'BTC-29MAR24', 'symbol': 'BTC/USD:USDC-240329', 'base': 'BTC', 'quote': 'USD', 'settle': 'USDC', 'baseId': 'BTC', 'quoteId': 'USD', 'settleId': 'USDC', 'type': 'future', 'spot': False, 'margin': None, 'swap': False, 'future': True, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': 1711699200000, 'expiryDatetime': '2024-03-29T08:00:00.000Z', 'strike': None, 'optionType': None, 'precision': {'amount': 0.001, 'price': 0.5}, 'limits': {'leverage': {'min': 1.0, 'max': 50.0}, 'amount': {'min': 0.001, 'max': 10.0}, 'price': {'min': 0.5, 'max': 999999.0}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'BTC-29MAR24', 'contractType': 'LinearFutures', 'status': 'Trading', 'baseCoin': 'BTC', 'quoteCoin': 'USD', 'launchTime': '1688716800000', 'deliveryTime': '1711699200000', 'deliveryFeeRate': '0', 'priceScale': '2', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '50.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.50', 'maxPrice': '999999.00', 'tickSize': '0.50'}, 'lotSizeFilter': {'maxOrderQty': '10.000', 'minOrderQty': '0.001', 'qtyStep': '0.001', 'postOnlyMaxOrderQty': '50.000'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDC', 'copyTrading': 'none'}}, {'id': 'BTC-29SEP23', 'symbol': 'BTC/USD:USDC-230929', 'base': 'BTC', 'quote': 'USD', 'settle': 'USDC', 'baseId': 'BTC', 'quoteId': 'USD', 'settleId': 'USDC', 'type': 'future', 'spot': False, 'margin': None, 'swap': False, 'future': True, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': 1695974400000, 'expiryDatetime': '2023-09-29T08:00:00.000Z', 'strike': None, 'optionType': None, 'precision': {'amount': 0.001, 'price': 0.5}, 'limits': {'leverage': {'min': 1.0, 'max': 50.0}, 'amount': {'min': 0.001, 'max': 10.0}, 'price': {'min': 0.5, 'max': 999999.0}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'BTC-29SEP23', 'contractType': 'LinearFutures', 'status': 'Trading', 'baseCoin': 'BTC', 'quoteCoin': 'USD', 'launchTime': '1679644800000', 'deliveryTime': '1695974400000', 'deliveryFeeRate': '0', 'priceScale': '2', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '50.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.50', 'maxPrice': '999999.00', 'tickSize': '0.50'}, 'lotSizeFilter': {'maxOrderQty': '10.000', 'minOrderQty': '0.001', 'qtyStep': '0.001', 'postOnlyMaxOrderQty': '50.000'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDC', 'copyTrading': 'none'}}, {'id': 'BTCPERP', 'symbol': 'BTC/USD:USDC', 'base': 'BTC', 'quote': 'USD', 'settle': 'USDC', 'baseId': 'BTC', 'quoteId': 'USD', 'settleId': 'USDC', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.001, 'price': 0.1}, 'limits': {'leverage': {'min': 1.0, 'max': 125.0}, 'amount': {'min': 0.001, 'max': 20.0}, 'price': {'min': 0.1, 'max': 199999.8}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'BTCPERP', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'BTC', 'quoteCoin': 'USD', 'launchTime': '1610604231000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '2', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '125.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.10', 'maxPrice': '199999.80', 'tickSize': '0.10'}, 'lotSizeFilter': {'maxOrderQty': '20.000', 'minOrderQty': '0.001', 'qtyStep': '0.001', 'postOnlyMaxOrderQty': '100.000'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDC', 'copyTrading': 'none'}}, {'id': 'BTCUSDT', 'symbol': 'BTC/USDT:USDT', 'base': 'BTC', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'BTC', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.001, 'price': 0.1}, 'limits': {'leverage': {'min': 1.0, 'max': 100.0}, 'amount': {'min': 0.001, 'max': 100.0}, 'price': {'min': 0.1, 'max': 199999.8}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'BTCUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'BTC', 'quoteCoin': 'USDT', 'launchTime': '1584230400000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '2', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '100.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.10', 'maxPrice': '199999.80', 'tickSize': '0.10'}, 'lotSizeFilter': {'maxOrderQty': '100.000', 'minOrderQty': '0.001', 'qtyStep': '0.001', 'postOnlyMaxOrderQty': '1000.000'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'normalOnly'}}, {'id': 'BUSDUSDT', 'symbol': 'BUSD/USDT:USDT', 'base': 'BUSD', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'BUSD', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.1, 'price': 0.0001}, 'limits': {'leverage': {'min': 1.0, 'max': 25.0}, 'amount': {'min': 0.1, 'max': 100000.0}, 'price': {'min': 0.0001, 'max': 199.9998}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'BUSDUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'BUSD', 'quoteCoin': 'USDT', 'launchTime': '1676340857000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '4', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '25.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.0001', 'maxPrice': '199.9998', 'tickSize': '0.0001'}, 'lotSizeFilter': {'maxOrderQty': '100000.0', 'minOrderQty': '0.1', 'qtyStep': '0.1', 'postOnlyMaxOrderQty': '500000.0'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'C98USDT', 'symbol': 'C98/USDT:USDT', 'base': 'C98', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'C98', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.1, 'price': 0.0005}, 'limits': {'leverage': {'min': 1.0, 'max': 25.0}, 'amount': {'min': 0.1, 'max': 31200.0}, 'price': {'min': 0.0005, 'max': 999.999}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'C98USDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'C98', 'quoteCoin': 'USDT', 'launchTime': '1635120000000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '4', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '25.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.0005', 'maxPrice': '999.9990', 'tickSize': '0.0005'}, 'lotSizeFilter': {'maxOrderQty': '31200.0', 'minOrderQty': '0.1', 'qtyStep': '0.1', 'postOnlyMaxOrderQty': '312000.0'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'CEEKUSDT', 'symbol': 'CEEK/USDT:USDT', 'base': 'CEEK', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'CEEK', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 1.0, 'price': 5e-05}, 'limits': {'leverage': {'min': 1.0, 'max': 25.0}, 'amount': {'min': 1.0, 'max': 18200.0}, 'price': {'min': 5e-05, 'max': 99.9999}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'CEEKUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'CEEK', 'quoteCoin': 'USDT', 'launchTime': '1658127698000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '5', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '25.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.00005', 'maxPrice': '99.99990', 'tickSize': '0.00005'}, 'lotSizeFilter': {'maxOrderQty': '18200', 'minOrderQty': '1', 'qtyStep': '1', 'postOnlyMaxOrderQty': '182000'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'CELOUSDT', 'symbol': 'CELO/USDT:USDT', 'base': 'CELO', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'CELO', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.1, 'price': 0.001}, 'limits': {'leverage': {'min': 1.0, 'max': 12.5}, 'amount': {'min': 0.1, 'max': 17400.0}, 'price': {'min': 0.001, 'max': 1999.998}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'CELOUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'CELO', 'quoteCoin': 'USDT', 'launchTime': '1641367582000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '3', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '12.50', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.001', 'maxPrice': '1999.998', 'tickSize': '0.001'}, 'lotSizeFilter': {'maxOrderQty': '17400.0', 'minOrderQty': '0.1', 'qtyStep': '0.1', 'postOnlyMaxOrderQty': '174000.0'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'CELRUSDT', 'symbol': 'CELR/USDT:USDT', 'base': 'CELR', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'CELR', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 1.0, 'price': 5e-05}, 'limits': {'leverage': {'min': 1.0, 'max': 25.0}, 'amount': {'min': 1.0, 'max': 775600.0}, 'price': {'min': 5e-05, 'max': 99.9999}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'CELRUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'CELR', 'quoteCoin': 'USDT', 'launchTime': '1635120000000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '5', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '25.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.00005', 'maxPrice': '99.99990', 'tickSize': '0.00005'}, 'lotSizeFilter': {'maxOrderQty': '775600', 'minOrderQty': '1', 'qtyStep': '1', 'postOnlyMaxOrderQty': '7756000'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'CFXUSDT', 'symbol': 'CFX/USDT:USDT', 'base': 'CFX', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'CFX', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 1.0, 'price': 1e-05}, 'limits': {'leverage': {'min': 1.0, 'max': 50.0}, 'amount': {'min': 1.0, 'max': 500000.0}, 'price': {'min': 1e-05, 'max': 19.99998}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'CFXUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'CFX', 'quoteCoin': 'USDT', 'launchTime': '1676883925000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '5', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '50.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.00001', 'maxPrice': '19.99998', 'tickSize': '0.00001'}, 'lotSizeFilter': {'maxOrderQty': '500000', 'minOrderQty': '1', 'qtyStep': '1', 'postOnlyMaxOrderQty': '2500000'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'normalOnly'}}, {'id': 'CHRUSDT', 'symbol': 'CHR/USDT:USDT', 'base': 'CHR', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'CHR', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.1, 'price': 0.0001}, 'limits': {'leverage': {'min': 1.0, 'max': 25.0}, 'amount': {'min': 0.1, 'max': 50200.0}, 'price': {'min': 0.0001, 'max': 199.9998}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'CHRUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'CHR', 'quoteCoin': 'USDT', 'launchTime': '1637193600000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '4', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '25.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.0001', 'maxPrice': '199.9998', 'tickSize': '0.0001'}, 'lotSizeFilter': {'maxOrderQty': '50200.0', 'minOrderQty': '0.1', 'qtyStep': '0.1', 'postOnlyMaxOrderQty': '502000.0'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'CHZUSDT', 'symbol': 'CHZ/USDT:USDT', 'base': 'CHZ', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'CHZ', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 1.0, 'price': 5e-05}, 'limits': {'leverage': {'min': 1.0, 'max': 25.0}, 'amount': {'min': 1.0, 'max': 700000.0}, 'price': {'min': 5e-05, 'max': 99.9999}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'CHZUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'CHZ', 'quoteCoin': 'USDT', 'launchTime': '1633996800000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '5', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '25.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.00005', 'maxPrice': '99.99990', 'tickSize': '0.00005'}, 'lotSizeFilter': {'maxOrderQty': '700000', 'minOrderQty': '1', 'qtyStep': '1', 'postOnlyMaxOrderQty': '7000000'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'normalOnly'}}, {'id': 'CKBUSDT', 'symbol': 'CKB/USDT:USDT', 'base': 'CKB', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'CKB', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 10.0, 'price': 5e-06}, 'limits': {'leverage': {'min': 1.0, 'max': 12.5}, 'amount': {'min': 10.0, 'max': 740000.0}, 'price': {'min': 5e-06, 'max': 9.99999}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'CKBUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'CKB', 'quoteCoin': 'USDT', 'launchTime': '1644901148000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '6', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '12.50', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.000005', 'maxPrice': '9.999990', 'tickSize': '0.000005'}, 'lotSizeFilter': {'maxOrderQty': '740000', 'minOrderQty': '10', 'qtyStep': '10', 'postOnlyMaxOrderQty': '7400000'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'COMBOUSDT', 'symbol': 'COMBO/USDT:USDT', 'base': 'COMBO', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'COMBO', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.1, 'price': 0.0005}, 'limits': {'leverage': {'min': 1.0, 'max': 12.5}, 'amount': {'min': 0.1, 'max': 18000.0}, 'price': {'min': 0.0005, 'max': 999.999}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'COMBOUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'COMBO', 'quoteCoin': 'USDT', 'launchTime': '1686210792000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '4', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '12.50', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.0005', 'maxPrice': '999.9990', 'tickSize': '0.0005'}, 'lotSizeFilter': {'maxOrderQty': '18000.0', 'minOrderQty': '0.1', 'qtyStep': '0.1', 'postOnlyMaxOrderQty': '90000.0'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'COMPUSDT', 'symbol': 'COMP/USDT:USDT', 'base': 'COMP', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'COMP', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.01, 'price': 0.05}, 'limits': {'leverage': {'min': 1.0, 'max': 25.0}, 'amount': {'min': 0.01, 'max': 640.0}, 'price': {'min': 0.05, 'max': 99999.9}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'COMPUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'COMP', 'quoteCoin': 'USDT', 'launchTime': '1628726400000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '2', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '25.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.05', 'maxPrice': '99999.90', 'tickSize': '0.05'}, 'lotSizeFilter': {'maxOrderQty': '640.00', 'minOrderQty': '0.01', 'qtyStep': '0.01', 'postOnlyMaxOrderQty': '6400.00'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'COREUSDT', 'symbol': 'CORE/USDT:USDT', 'base': 'CORE', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'CORE', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.1, 'price': 0.0005}, 'limits': {'leverage': {'min': 1.0, 'max': 25.0}, 'amount': {'min': 0.1, 'max': 10000.0}, 'price': {'min': 0.0005, 'max': 999.999}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'COREUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'CORE', 'quoteCoin': 'USDT', 'launchTime': '1675927514000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '4', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '25.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.0005', 'maxPrice': '999.9990', 'tickSize': '0.0005'}, 'lotSizeFilter': {'maxOrderQty': '10000.0', 'minOrderQty': '0.1', 'qtyStep': '0.1', 'postOnlyMaxOrderQty': '50000.0'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'COTIUSDT', 'symbol': 'COTI/USDT:USDT', 'base': 'COTI', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'COTI', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 1.0, 'price': 1e-05}, 'limits': {'leverage': {'min': 1.0, 'max': 16.67}, 'amount': {'min': 1.0, 'max': 182000.0}, 'price': {'min': 1e-05, 'max': 19.99998}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'COTIUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'COTI', 'quoteCoin': 'USDT', 'launchTime': '1638445117000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '5', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '16.67', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.00001', 'maxPrice': '19.99998', 'tickSize': '0.00001'}, 'lotSizeFilter': {'maxOrderQty': '182000', 'minOrderQty': '1', 'qtyStep': '1', 'postOnlyMaxOrderQty': '1820000'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'CROUSDT', 'symbol': 'CRO/USDT:USDT', 'base': 'CRO', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'CRO', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 1.0, 'price': 5e-05}, 'limits': {'leverage': {'min': 1.0, 'max': 25.0}, 'amount': {'min': 1.0, 'max': 90000.0}, 'price': {'min': 5e-05, 'max': 99.9999}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'CROUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'CRO', 'quoteCoin': 'USDT', 'launchTime': '1637625600000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '5', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '25.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.00005', 'maxPrice': '99.99990', 'tickSize': '0.00005'}, 'lotSizeFilter': {'maxOrderQty': '90000', 'minOrderQty': '1', 'qtyStep': '1', 'postOnlyMaxOrderQty': '900000'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'CRVUSDT', 'symbol': 'CRV/USDT:USDT', 'base': 'CRV', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'CRV', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.1, 'price': 0.001}, 'limits': {'leverage': {'min': 1.0, 'max': 25.0}, 'amount': {'min': 0.1, 'max': 70000.0}, 'price': {'min': 0.001, 'max': 1999.998}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'CRVUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'CRV', 'quoteCoin': 'USDT', 'launchTime': '1635120000000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '3', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '25.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.001', 'maxPrice': '1999.998', 'tickSize': '0.001'}, 'lotSizeFilter': {'maxOrderQty': '70000.0', 'minOrderQty': '0.1', 'qtyStep': '0.1', 'postOnlyMaxOrderQty': '700000.0'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'normalOnly'}}, {'id': 'CTCUSDT', 'symbol': 'CTC/USDT:USDT', 'base': 'CTC', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'CTC', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 1.0, 'price': 5e-05}, 'limits': {'leverage': {'min': 1.0, 'max': 12.5}, 'amount': {'min': 1.0, 'max': 3500.0}, 'price': {'min': 5e-05, 'max': 99.9999}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'CTCUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'CTC', 'quoteCoin': 'USDT', 'launchTime': '1652774270000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '5', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '12.50', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.00005', 'maxPrice': '99.99990', 'tickSize': '0.00005'}, 'lotSizeFilter': {'maxOrderQty': '3500', 'minOrderQty': '1', 'qtyStep': '1', 'postOnlyMaxOrderQty': '35000'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'CTKUSDT', 'symbol': 'CTK/USDT:USDT', 'base': 'CTK', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'CTK', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.1, 'price': 0.0001}, 'limits': {'leverage': {'min': 1.0, 'max': 16.67}, 'amount': {'min': 0.1, 'max': 17200.0}, 'price': {'min': 0.0001, 'max': 199.9998}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'CTKUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'CTK', 'quoteCoin': 'USDT', 'launchTime': '1638444847000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '4', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '16.67', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.0001', 'maxPrice': '199.9998', 'tickSize': '0.0001'}, 'lotSizeFilter': {'maxOrderQty': '17200.0', 'minOrderQty': '0.1', 'qtyStep': '0.1', 'postOnlyMaxOrderQty': '172000.0'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'CTSIUSDT', 'symbol': 'CTSI/USDT:USDT', 'base': 'CTSI', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'CTSI', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 1.0, 'price': 5e-05}, 'limits': {'leverage': {'min': 1.0, 'max': 25.0}, 'amount': {'min': 1.0, 'max': 156000.0}, 'price': {'min': 5e-05, 'max': 99.9999}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'CTSIUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'CTSI', 'quoteCoin': 'USDT', 'launchTime': '1648018511000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '5', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '25.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.00005', 'maxPrice': '99.99990', 'tickSize': '0.00005'}, 'lotSizeFilter': {'maxOrderQty': '156000', 'minOrderQty': '1', 'qtyStep': '1', 'postOnlyMaxOrderQty': '1560000'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'CVCUSDT', 'symbol': 'CVC/USDT:USDT', 'base': 'CVC', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'CVC', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 1.0, 'price': 1e-05}, 'limits': {'leverage': {'min': 1.0, 'max': 12.5}, 'amount': {'min': 1.0, 'max': 91800.0}, 'price': {'min': 1e-05, 'max': 19.99998}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'CVCUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'CVC', 'quoteCoin': 'USDT', 'launchTime': '1638285651000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '5', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '12.50', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.00001', 'maxPrice': '19.99998', 'tickSize': '0.00001'}, 'lotSizeFilter': {'maxOrderQty': '91800', 'minOrderQty': '1', 'qtyStep': '1', 'postOnlyMaxOrderQty': '918000'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'CVXUSDT', 'symbol': 'CVX/USDT:USDT', 'base': 'CVX', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'CVX', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.01, 'price': 0.005}, 'limits': {'leverage': {'min': 1.0, 'max': 25.0}, 'amount': {'min': 0.01, 'max': 1200.0}, 'price': {'min': 0.005, 'max': 9999.99}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'CVXUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'CVX', 'quoteCoin': 'USDT', 'launchTime': '1650440282000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '3', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '25.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.005', 'maxPrice': '9999.990', 'tickSize': '0.005'}, 'lotSizeFilter': {'maxOrderQty': '1200.00', 'minOrderQty': '0.01', 'qtyStep': '0.01', 'postOnlyMaxOrderQty': '12000.00'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'DARUSDT', 'symbol': 'DAR/USDT:USDT', 'base': 'DAR', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'DAR', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.1, 'price': 0.0005}, 'limits': {'leverage': {'min': 1.0, 'max': 25.0}, 'amount': {'min': 0.1, 'max': 27525.0}, 'price': {'min': 0.0005, 'max': 999.999}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'DARUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'DAR', 'quoteCoin': 'USDT', 'launchTime': '1651467113000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '4', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '25.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.0005', 'maxPrice': '999.9990', 'tickSize': '0.0005'}, 'lotSizeFilter': {'maxOrderQty': '27525.0', 'minOrderQty': '0.1', 'qtyStep': '0.1', 'postOnlyMaxOrderQty': '275250.0'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'DASHUSDT', 'symbol': 'DASH/USDT:USDT', 'base': 'DASH', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'DASH', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.01, 'price': 0.05}, 'limits': {'leverage': {'min': 1.0, 'max': 25.0}, 'amount': {'min': 0.01, 'max': 410.0}, 'price': {'min': 0.05, 'max': 99999.9}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'DASHUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'DASH', 'quoteCoin': 'USDT', 'launchTime': '1633996800000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '2', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '25.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.05', 'maxPrice': '99999.90', 'tickSize': '0.05'}, 'lotSizeFilter': {'maxOrderQty': '410.00', 'minOrderQty': '0.01', 'qtyStep': '0.01', 'postOnlyMaxOrderQty': '4100.00'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'DENTUSDT', 'symbol': 'DENT/USDT:USDT', 'base': 'DENT', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'DENT', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 100.0, 'price': 1e-06}, 'limits': {'leverage': {'min': 1.0, 'max': 12.5}, 'amount': {'min': 100.0, 'max': 9800000.0}, 'price': {'min': 1e-06, 'max': 1.999998}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'DENTUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'DENT', 'quoteCoin': 'USDT', 'launchTime': '1642068928000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '6', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '12.50', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.000001', 'maxPrice': '1.999998', 'tickSize': '0.000001'}, 'lotSizeFilter': {'maxOrderQty': '9800000', 'minOrderQty': '100', 'qtyStep': '100', 'postOnlyMaxOrderQty': '98000000'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'DGBUSDT', 'symbol': 'DGB/USDT:USDT', 'base': 'DGB', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'DGB', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 10.0, 'price': 1e-05}, 'limits': {'leverage': {'min': 1.0, 'max': 25.0}, 'amount': {'min': 10.0, 'max': 351400.0}, 'price': {'min': 1e-05, 'max': 19.99998}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'DGBUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'DGB', 'quoteCoin': 'USDT', 'launchTime': '1648452331000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '5', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '25.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.00001', 'maxPrice': '19.99998', 'tickSize': '0.00001'}, 'lotSizeFilter': {'maxOrderQty': '351400', 'minOrderQty': '10', 'qtyStep': '10', 'postOnlyMaxOrderQty': '3514000'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'DODOUSDT', 'symbol': 'DODO/USDT:USDT', 'base': 'DODO', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'DODO', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 1.0, 'price': 5e-05}, 'limits': {'leverage': {'min': 1.0, 'max': 25.0}, 'amount': {'min': 1.0, 'max': 46000.0}, 'price': {'min': 5e-05, 'max': 99.9999}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'DODOUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'DODO', 'quoteCoin': 'USDT', 'launchTime': '1650457800000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '5', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '25.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.00005', 'maxPrice': '99.99990', 'tickSize': '0.00005'}, 'lotSizeFilter': {'maxOrderQty': '46000', 'minOrderQty': '1', 'qtyStep': '1', 'postOnlyMaxOrderQty': '460000'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'DOGEUSDT', 'symbol': 'DOGE/USDT:USDT', 'base': 'DOGE', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'DOGE', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 1.0, 'price': 1e-05}, 'limits': {'leverage': {'min': 1.0, 'max': 75.0}, 'amount': {'min': 1.0, 'max': 10000000.0}, 'price': {'min': 1e-05, 'max': 19.99998}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'DOGEUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'DOGE', 'quoteCoin': 'USDT', 'launchTime': '0', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '5', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '75.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.00001', 'maxPrice': '19.99998', 'tickSize': '0.00001'}, 'lotSizeFilter': {'maxOrderQty': '10000000', 'minOrderQty': '1', 'qtyStep': '1', 'postOnlyMaxOrderQty': '100000000'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'normalOnly'}}, {'id': 'DOTUSDT', 'symbol': 'DOT/USDT:USDT', 'base': 'DOT', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'DOT', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.1, 'price': 0.001}, 'limits': {'leverage': {'min': 1.0, 'max': 50.0}, 'amount': {'min': 0.1, 'max': 32900.0}, 'price': {'min': 0.001, 'max': 1999.998}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'DOTUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'DOT', 'quoteCoin': 'USDT', 'launchTime': '0', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '3', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '50.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.001', 'maxPrice': '1999.998', 'tickSize': '0.001'}, 'lotSizeFilter': {'maxOrderQty': '32900.0', 'minOrderQty': '0.1', 'qtyStep': '0.1', 'postOnlyMaxOrderQty': '329000.0'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'normalOnly'}}, {'id': 'DUSKUSDT', 'symbol': 'DUSK/USDT:USDT', 'base': 'DUSK', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'DUSK', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 1.0, 'price': 1e-05}, 'limits': {'leverage': {'min': 1.0, 'max': 12.5}, 'amount': {'min': 1.0, 'max': 52500.0}, 'price': {'min': 1e-05, 'max': 19.99998}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'DUSKUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'DUSK', 'quoteCoin': 'USDT', 'launchTime': '1639673130000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '5', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '12.50', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.00001', 'maxPrice': '19.99998', 'tickSize': '0.00001'}, 'lotSizeFilter': {'maxOrderQty': '52500', 'minOrderQty': '1', 'qtyStep': '1', 'postOnlyMaxOrderQty': '525000'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'DYDXUSDT', 'symbol': 'DYDX/USDT:USDT', 'base': 'DYDX', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'DYDX', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.1, 'price': 0.001}, 'limits': {'leverage': {'min': 1.0, 'max': 25.0}, 'amount': {'min': 0.1, 'max': 45000.0}, 'price': {'min': 0.001, 'max': 1999.998}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'DYDXUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'DYDX', 'quoteCoin': 'USDT', 'launchTime': '1633910400000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '3', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '25.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.001', 'maxPrice': '1999.998', 'tickSize': '0.001'}, 'lotSizeFilter': {'maxOrderQty': '45000.0', 'minOrderQty': '0.1', 'qtyStep': '0.1', 'postOnlyMaxOrderQty': '450000.0'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'normalOnly'}}, {'id': 'EDUUSDT', 'symbol': 'EDU/USDT:USDT', 'base': 'EDU', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'EDU', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.1, 'price': 0.0001}, 'limits': {'leverage': {'min': 1.0, 'max': 25.0}, 'amount': {'min': 0.1, 'max': 20000.0}, 'price': {'min': 0.0001, 'max': 199.9998}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'EDUUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'EDU', 'quoteCoin': 'USDT', 'launchTime': '1683273334000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '4', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '25.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.0001', 'maxPrice': '199.9998', 'tickSize': '0.0001'}, 'lotSizeFilter': {'maxOrderQty': '20000.0', 'minOrderQty': '0.1', 'qtyStep': '0.1', 'postOnlyMaxOrderQty': '100000.0'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'normalOnly'}}, {'id': 'EGLDUSDT', 'symbol': 'EGLD/USDT:USDT', 'base': 'EGLD', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'EGLD', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.01, 'price': 0.05}, 'limits': {'leverage': {'min': 1.0, 'max': 25.0}, 'amount': {'min': 0.01, 'max': 1300.0}, 'price': {'min': 0.05, 'max': 99999.9}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'EGLDUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'EGLD', 'quoteCoin': 'USDT', 'launchTime': '1637596800000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '2', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '25.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.05', 'maxPrice': '99999.90', 'tickSize': '0.05'}, 'lotSizeFilter': {'maxOrderQty': '1300.00', 'minOrderQty': '0.01', 'qtyStep': '0.01', 'postOnlyMaxOrderQty': '13000.00'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'ENJUSDT', 'symbol': 'ENJ/USDT:USDT', 'base': 'ENJ', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'ENJ', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.1, 'price': 0.0005}, 'limits': {'leverage': {'min': 1.0, 'max': 25.0}, 'amount': {'min': 0.1, 'max': 116000.0}, 'price': {'min': 0.0005, 'max': 999.999}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'ENJUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'ENJ', 'quoteCoin': 'USDT', 'launchTime': '1635696000000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '4', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '25.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.0005', 'maxPrice': '999.9990', 'tickSize': '0.0005'}, 'lotSizeFilter': {'maxOrderQty': '116000.0', 'minOrderQty': '0.1', 'qtyStep': '0.1', 'postOnlyMaxOrderQty': '1160000.0'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'ENSUSDT', 'symbol': 'ENS/USDT:USDT', 'base': 'ENS', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'ENS', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.1, 'price': 0.01}, 'limits': {'leverage': {'min': 1.0, 'max': 25.0}, 'amount': {'min': 0.1, 'max': 1600.0}, 'price': {'min': 0.01, 'max': 19999.98}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'ENSUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'ENS', 'quoteCoin': 'USDT', 'launchTime': '1636675200000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '2', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '25.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.01', 'maxPrice': '19999.98', 'tickSize': '0.01'}, 'lotSizeFilter': {'maxOrderQty': '1600.0', 'minOrderQty': '0.1', 'qtyStep': '0.1', 'postOnlyMaxOrderQty': '16000.0'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'normalOnly'}}, {'id': 'EOSUSDT', 'symbol': 'EOS/USDT:USDT', 'base': 'EOS', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'EOS', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.1, 'price': 0.001}, 'limits': {'leverage': {'min': 1.0, 'max': 50.0}, 'amount': {'min': 0.1, 'max': 55000.0}, 'price': {'min': 0.001, 'max': 1999.998}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'EOSUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'EOS', 'quoteCoin': 'USDT', 'launchTime': '0', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '3', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '50.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.001', 'maxPrice': '1999.998', 'tickSize': '0.001'}, 'lotSizeFilter': {'maxOrderQty': '55000.0', 'minOrderQty': '0.1', 'qtyStep': '0.1', 'postOnlyMaxOrderQty': '550000.0'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'normalOnly'}}, {'id': 'ETCPERP', 'symbol': 'ETC/USD:USDC', 'base': 'ETC', 'quote': 'USD', 'settle': 'USDC', 'baseId': 'ETC', 'quoteId': 'USD', 'settleId': 'USDC', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.1, 'price': 0.005}, 'limits': {'leverage': {'min': 1.0, 'max': 50.0}, 'amount': {'min': 0.1, 'max': 500.0}, 'price': {'min': 0.005, 'max': 9999.99}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'ETCPERP', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'ETC', 'quoteCoin': 'USD', 'launchTime': '1661990400000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '3', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '50.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.005', 'maxPrice': '9999.990', 'tickSize': '0.005'}, 'lotSizeFilter': {'maxOrderQty': '500.0', 'minOrderQty': '0.1', 'qtyStep': '0.1', 'postOnlyMaxOrderQty': '2500.0'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDC', 'copyTrading': 'none'}}, {'id': 'ETCUSDT', 'symbol': 'ETC/USDT:USDT', 'base': 'ETC', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'ETC', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.1, 'price': 0.005}, 'limits': {'leverage': {'min': 1.0, 'max': 50.0}, 'amount': {'min': 0.1, 'max': 10000.0}, 'price': {'min': 0.005, 'max': 9999.99}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'ETCUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'ETC', 'quoteCoin': 'USDT', 'launchTime': '0', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '3', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '50.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.005', 'maxPrice': '9999.990', 'tickSize': '0.005'}, 'lotSizeFilter': {'maxOrderQty': '10000.0', 'minOrderQty': '0.1', 'qtyStep': '0.1', 'postOnlyMaxOrderQty': '100000.0'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'normalOnly'}}, {'id': 'ETH-04AUG23', 'symbol': 'ETH/USD:USDC-230804', 'base': 'ETH', 'quote': 'USD', 'settle': 'USDC', 'baseId': 'ETH', 'quoteId': 'USD', 'settleId': 'USDC', 'type': 'future', 'spot': False, 'margin': None, 'swap': False, 'future': True, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': 1691136000000, 'expiryDatetime': '2023-08-04T08:00:00.000Z', 'strike': None, 'optionType': None, 'precision': {'amount': 0.01, 'price': 0.05}, 'limits': {'leverage': {'min': 1.0, 'max': 50.0}, 'amount': {'min': 0.01, 'max': 150.0}, 'price': {'min': 0.05, 'max': 99999.9}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'ETH-04AUG23', 'contractType': 'LinearFutures', 'status': 'Trading', 'baseCoin': 'ETH', 'quoteCoin': 'USD', 'launchTime': '1689321600000', 'deliveryTime': '1691136000000', 'deliveryFeeRate': '0', 'priceScale': '2', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '50.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.05', 'maxPrice': '99999.90', 'tickSize': '0.05'}, 'lotSizeFilter': {'maxOrderQty': '150.00', 'minOrderQty': '0.01', 'qtyStep': '0.01', 'postOnlyMaxOrderQty': '750.00'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDC', 'copyTrading': 'none'}}, {'id': 'ETH-11AUG23', 'symbol': 'ETH/USD:USDC-230811', 'base': 'ETH', 'quote': 'USD', 'settle': 'USDC', 'baseId': 'ETH', 'quoteId': 'USD', 'settleId': 'USDC', 'type': 'future', 'spot': False, 'margin': None, 'swap': False, 'future': True, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': 1691740800000, 'expiryDatetime': '2023-08-11T08:00:00.000Z', 'strike': None, 'optionType': None, 'precision': {'amount': 0.01, 'price': 0.05}, 'limits': {'leverage': {'min': 1.0, 'max': 50.0}, 'amount': {'min': 0.01, 'max': 150.0}, 'price': {'min': 0.05, 'max': 99999.9}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'ETH-11AUG23', 'contractType': 'LinearFutures', 'status': 'Trading', 'baseCoin': 'ETH', 'quoteCoin': 'USD', 'launchTime': '1689926400000', 'deliveryTime': '1691740800000', 'deliveryFeeRate': '0', 'priceScale': '2', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '50.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.05', 'maxPrice': '99999.90', 'tickSize': '0.05'}, 'lotSizeFilter': {'maxOrderQty': '150.00', 'minOrderQty': '0.01', 'qtyStep': '0.01', 'postOnlyMaxOrderQty': '750.00'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDC', 'copyTrading': 'none'}}, {'id': 'ETH-25AUG23', 'symbol': 'ETH/USD:USDC-230825', 'base': 'ETH', 'quote': 'USD', 'settle': 'USDC', 'baseId': 'ETH', 'quoteId': 'USD', 'settleId': 'USDC', 'type': 'future', 'spot': False, 'margin': None, 'swap': False, 'future': True, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': 1692950400000, 'expiryDatetime': '2023-08-25T08:00:00.000Z', 'strike': None, 'optionType': None, 'precision': {'amount': 0.01, 'price': 0.05}, 'limits': {'leverage': {'min': 1.0, 'max': 50.0}, 'amount': {'min': 0.01, 'max': 150.0}, 'price': {'min': 0.05, 'max': 99999.9}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'ETH-25AUG23', 'contractType': 'LinearFutures', 'status': 'Trading', 'baseCoin': 'ETH', 'quoteCoin': 'USD', 'launchTime': '1686297600000', 'deliveryTime': '1692950400000', 'deliveryFeeRate': '0', 'priceScale': '2', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '50.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.05', 'maxPrice': '99999.90', 'tickSize': '0.05'}, 'lotSizeFilter': {'maxOrderQty': '150.00', 'minOrderQty': '0.01', 'qtyStep': '0.01', 'postOnlyMaxOrderQty': '750.00'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDC', 'copyTrading': 'none'}}, {'id': 'ETH-28JUL23', 'symbol': 'ETH/USD:USDC-230728', 'base': 'ETH', 'quote': 'USD', 'settle': 'USDC', 'baseId': 'ETH', 'quoteId': 'USD', 'settleId': 'USDC', 'type': 'future', 'spot': False, 'margin': None, 'swap': False, 'future': True, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': 1690531200000, 'expiryDatetime': '2023-07-28T08:00:00.000Z', 'strike': None, 'optionType': None, 'precision': {'amount': 0.01, 'price': 0.05}, 'limits': {'leverage': {'min': 1.0, 'max': 50.0}, 'amount': {'min': 0.01, 'max': 150.0}, 'price': {'min': 0.05, 'max': 99999.9}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'ETH-28JUL23', 'contractType': 'LinearFutures', 'status': 'Trading', 'baseCoin': 'ETH', 'quoteCoin': 'USD', 'launchTime': '1683273600000', 'deliveryTime': '1690531200000', 'deliveryFeeRate': '0', 'priceScale': '2', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '50.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.05', 'maxPrice': '99999.90', 'tickSize': '0.05'}, 'lotSizeFilter': {'maxOrderQty': '150.00', 'minOrderQty': '0.01', 'qtyStep': '0.01', 'postOnlyMaxOrderQty': '750.00'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDC', 'copyTrading': 'none'}}, {'id': 'ETH-29DEC23', 'symbol': 'ETH/USD:USDC-231229', 'base': 'ETH', 'quote': 'USD', 'settle': 'USDC', 'baseId': 'ETH', 'quoteId': 'USD', 'settleId': 'USDC', 'type': 'future', 'spot': False, 'margin': None, 'swap': False, 'future': True, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': 1703836800000, 'expiryDatetime': '2023-12-29T08:00:00.000Z', 'strike': None, 'optionType': None, 'precision': {'amount': 0.01, 'price': 0.05}, 'limits': {'leverage': {'min': 1.0, 'max': 50.0}, 'amount': {'min': 0.01, 'max': 150.0}, 'price': {'min': 0.05, 'max': 99999.9}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'ETH-29DEC23', 'contractType': 'LinearFutures', 'status': 'Trading', 'baseCoin': 'ETH', 'quoteCoin': 'USD', 'launchTime': '1680854400000', 'deliveryTime': '1703836800000', 'deliveryFeeRate': '0', 'priceScale': '2', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '50.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.05', 'maxPrice': '99999.90', 'tickSize': '0.05'}, 'lotSizeFilter': {'maxOrderQty': '150.00', 'minOrderQty': '0.01', 'qtyStep': '0.01', 'postOnlyMaxOrderQty': '750.00'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDC', 'copyTrading': 'none'}}, {'id': 'ETH-29MAR24', 'symbol': 'ETH/USD:USDC-240329', 'base': 'ETH', 'quote': 'USD', 'settle': 'USDC', 'baseId': 'ETH', 'quoteId': 'USD', 'settleId': 'USDC', 'type': 'future', 'spot': False, 'margin': None, 'swap': False, 'future': True, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': 1711699200000, 'expiryDatetime': '2024-03-29T08:00:00.000Z', 'strike': None, 'optionType': None, 'precision': {'amount': 0.01, 'price': 0.05}, 'limits': {'leverage': {'min': 1.0, 'max': 50.0}, 'amount': {'min': 0.01, 'max': 150.0}, 'price': {'min': 0.05, 'max': 99999.9}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'ETH-29MAR24', 'contractType': 'LinearFutures', 'status': 'Trading', 'baseCoin': 'ETH', 'quoteCoin': 'USD', 'launchTime': '1688716800000', 'deliveryTime': '1711699200000', 'deliveryFeeRate': '0', 'priceScale': '2', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '50.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.05', 'maxPrice': '99999.90', 'tickSize': '0.05'}, 'lotSizeFilter': {'maxOrderQty': '150.00', 'minOrderQty': '0.01', 'qtyStep': '0.01', 'postOnlyMaxOrderQty': '750.00'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDC', 'copyTrading': 'none'}}, {'id': 'ETH-29SEP23', 'symbol': 'ETH/USD:USDC-230929', 'base': 'ETH', 'quote': 'USD', 'settle': 'USDC', 'baseId': 'ETH', 'quoteId': 'USD', 'settleId': 'USDC', 'type': 'future', 'spot': False, 'margin': None, 'swap': False, 'future': True, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': 1695974400000, 'expiryDatetime': '2023-09-29T08:00:00.000Z', 'strike': None, 'optionType': None, 'precision': {'amount': 0.01, 'price': 0.05}, 'limits': {'leverage': {'min': 1.0, 'max': 50.0}, 'amount': {'min': 0.01, 'max': 150.0}, 'price': {'min': 0.05, 'max': 99999.9}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'ETH-29SEP23', 'contractType': 'LinearFutures', 'status': 'Trading', 'baseCoin': 'ETH', 'quoteCoin': 'USD', 'launchTime': '1679644800000', 'deliveryTime': '1695974400000', 'deliveryFeeRate': '0', 'priceScale': '2', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '50.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.05', 'maxPrice': '99999.90', 'tickSize': '0.05'}, 'lotSizeFilter': {'maxOrderQty': '150.00', 'minOrderQty': '0.01', 'qtyStep': '0.01', 'postOnlyMaxOrderQty': '750.00'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDC', 'copyTrading': 'none'}}, {'id': 'ETHPERP', 'symbol': 'ETH/USD:USDC', 'base': 'ETH', 'quote': 'USD', 'settle': 'USDC', 'baseId': 'ETH', 'quoteId': 'USD', 'settleId': 'USDC', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.01, 'price': 0.01}, 'limits': {'leverage': {'min': 1.0, 'max': 66.67}, 'amount': {'min': 0.01, 'max': 400.0}, 'price': {'min': 0.01, 'max': 19999.98}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'ETHPERP', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'ETH', 'quoteCoin': 'USD', 'launchTime': '1659657600000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '2', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '66.67', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.01', 'maxPrice': '19999.98', 'tickSize': '0.01'}, 'lotSizeFilter': {'maxOrderQty': '400.00', 'minOrderQty': '0.01', 'qtyStep': '0.01', 'postOnlyMaxOrderQty': '2000.00'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDC', 'copyTrading': 'none'}}, {'id': 'ETHUSDT', 'symbol': 'ETH/USDT:USDT', 'base': 'ETH', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'ETH', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.01, 'price': 0.01}, 'limits': {'leverage': {'min': 1.0, 'max': 100.0}, 'amount': {'min': 0.01, 'max': 1500.0}, 'price': {'min': 0.01, 'max': 19999.98}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'ETHUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'ETH', 'quoteCoin': 'USDT', 'launchTime': '1615766400000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '2', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '100.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.01', 'maxPrice': '19999.98', 'tickSize': '0.01'}, 'lotSizeFilter': {'maxOrderQty': '1500.00', 'minOrderQty': '0.01', 'qtyStep': '0.01', 'postOnlyMaxOrderQty': '15000.00'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'normalOnly'}}, {'id': 'ETHWUSDT', 'symbol': 'ETHW/USDT:USDT', 'base': 'ETHW', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'ETHW', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.01, 'price': 0.001}, 'limits': {'leverage': {'min': 1.0, 'max': 25.0}, 'amount': {'min': 0.01, 'max': 7500.0}, 'price': {'min': 0.001, 'max': 1999.998}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'ETHWUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'ETHW', 'quoteCoin': 'USDT', 'launchTime': '1663286400000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '3', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '25.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.001', 'maxPrice': '1999.998', 'tickSize': '0.001'}, 'lotSizeFilter': {'maxOrderQty': '7500.00', 'minOrderQty': '0.01', 'qtyStep': '0.01', 'postOnlyMaxOrderQty': '75000.00'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'normalOnly'}}, {'id': 'FETUSDT', 'symbol': 'FET/USDT:USDT', 'base': 'FET', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'FET', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 1.0, 'price': 5e-05}, 'limits': {'leverage': {'min': 1.0, 'max': 25.0}, 'amount': {'min': 1.0, 'max': 250000.0}, 'price': {'min': 5e-05, 'max': 99.9999}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'FETUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'FET', 'quoteCoin': 'USDT', 'launchTime': '1675759851000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '5', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '25.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.00005', 'maxPrice': '99.99990', 'tickSize': '0.00005'}, 'lotSizeFilter': {'maxOrderQty': '250000', 'minOrderQty': '1', 'qtyStep': '1', 'postOnlyMaxOrderQty': '1250000'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'FILUSDT', 'symbol': 'FIL/USDT:USDT', 'base': 'FIL', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'FIL', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.1, 'price': 0.001}, 'limits': {'leverage': {'min': 1.0, 'max': 25.0}, 'amount': {'min': 0.1, 'max': 45000.0}, 'price': {'min': 0.001, 'max': 1999.998}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'FILUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'FIL', 'quoteCoin': 'USDT', 'launchTime': '0', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '3', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '25.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.001', 'maxPrice': '1999.998', 'tickSize': '0.001'}, 'lotSizeFilter': {'maxOrderQty': '45000.0', 'minOrderQty': '0.1', 'qtyStep': '0.1', 'postOnlyMaxOrderQty': '450000.0'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'normalOnly'}}, {'id': 'FITFIUSDT', 'symbol': 'FITFI/USDT:USDT', 'base': 'FITFI', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'FITFI', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 1.0, 'price': 5e-06}, 'limits': {'leverage': {'min': 1.0, 'max': 50.0}, 'amount': {'min': 1.0, 'max': 200000.0}, 'price': {'min': 5e-06, 'max': 9.99999}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'FITFIUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'FITFI', 'quoteCoin': 'USDT', 'launchTime': '1652153037000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '6', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '50.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.000005', 'maxPrice': '9.999990', 'tickSize': '0.000005'}, 'lotSizeFilter': {'maxOrderQty': '200000', 'minOrderQty': '1', 'qtyStep': '1', 'postOnlyMaxOrderQty': '2000000'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'FLMUSDT', 'symbol': 'FLM/USDT:USDT', 'base': 'FLM', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'FLM', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 1.0, 'price': 5e-05}, 'limits': {'leverage': {'min': 1.0, 'max': 25.0}, 'amount': {'min': 1.0, 'max': 72500.0}, 'price': {'min': 5e-05, 'max': 99.9999}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'FLMUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'FLM', 'quoteCoin': 'USDT', 'launchTime': '1651467612000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '5', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '25.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.00005', 'maxPrice': '99.99990', 'tickSize': '0.00005'}, 'lotSizeFilter': {'maxOrderQty': '72500', 'minOrderQty': '1', 'qtyStep': '1', 'postOnlyMaxOrderQty': '725000'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'FLOWUSDT', 'symbol': 'FLOW/USDT:USDT', 'base': 'FLOW', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'FLOW', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.1, 'price': 0.0005}, 'limits': {'leverage': {'min': 1.0, 'max': 25.0}, 'amount': {'min': 0.1, 'max': 23000.0}, 'price': {'min': 0.0005, 'max': 999.999}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'FLOWUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'FLOW', 'quoteCoin': 'USDT', 'launchTime': '1637856000000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '4', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '25.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.0005', 'maxPrice': '999.9990', 'tickSize': '0.0005'}, 'lotSizeFilter': {'maxOrderQty': '23000.0', 'minOrderQty': '0.1', 'qtyStep': '0.1', 'postOnlyMaxOrderQty': '230000.0'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'FLRUSDT', 'symbol': 'FLR/USDT:USDT', 'base': 'FLR', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'FLR', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 10.0, 'price': 1e-05}, 'limits': {'leverage': {'min': 1.0, 'max': 25.0}, 'amount': {'min': 10.0, 'max': 379680.0}, 'price': {'min': 1e-05, 'max': 19.99998}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'FLRUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'FLR', 'quoteCoin': 'USDT', 'launchTime': '1678953508000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '5', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '25.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.00001', 'maxPrice': '19.99998', 'tickSize': '0.00001'}, 'lotSizeFilter': {'maxOrderQty': '379680', 'minOrderQty': '10', 'qtyStep': '10', 'postOnlyMaxOrderQty': '1898400'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}, {'id': 'FTMUSDT', 'symbol': 'FTM/USDT:USDT', 'base': 'FTM', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'FTM', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 1.0, 'price': 0.0001}, 'limits': {'leverage': {'min': 1.0, 'max': 75.0}, 'amount': {'min': 1.0, 'max': 360000.0}, 'price': {'min': 0.0001, 'max': 199.9998}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'FTMUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'FTM', 'quoteCoin': 'USDT', 'launchTime': '1632355200000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '4', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '75.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.0001', 'maxPrice': '199.9998', 'tickSize': '0.0001'}, 'lotSizeFilter': {'maxOrderQty': '360000', 'minOrderQty': '1', 'qtyStep': '1', 'postOnlyMaxOrderQty': '3600000'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'normalOnly'}}, {'id': 'FXSUSDT', 'symbol': 'FXS/USDT:USDT', 'base': 'FXS', 'quote': 'USDT', 'settle': 'USDT', 'baseId': 'FXS', 'quoteId': 'USDT', 'settleId': 'USDT', 'type': 'swap', 'spot': False, 'margin': None, 'swap': True, 'future': False, 'option': False, 'active': True, 'contract': True, 'linear': True, 'inverse': False, 'taker': 0.0006, 'maker': 0.0001, 'contractSize': 1.0, 'expiry': None, 'expiryDatetime': None, 'strike': None, 'optionType': None, 'precision': {'amount': 0.01, 'price': 0.0005}, 'limits': {'leverage': {'min': 1.0, 'max': 25.0}, 'amount': {'min': 0.01, 'max': 2400.0}, 'price': {'min': 0.0005, 'max': 999.999}, 'cost': {'min': None, 'max': None}}, 'info': {'symbol': 'FXSUSDT', 'contractType': 'LinearPerpetual', 'status': 'Trading', 'baseCoin': 'FXS', 'quoteCoin': 'USDT', 'launchTime': '1649044820000', 'deliveryTime': '0', 'deliveryFeeRate': '', 'priceScale': '4', 'leverageFilter': {'minLeverage': '1', 'maxLeverage': '25.00', 'leverageStep': '0.01'}, 'priceFilter': {'minPrice': '0.0005', 'maxPrice': '999.9990', 'tickSize': '0.0005'}, 'lotSizeFilter': {'maxOrderQty': '2400.00', 'minOrderQty': '0.01', 'qtyStep': '0.01', 'postOnlyMaxOrderQty': '24000.00'}, 'unifiedMarginTrade': True, 'fundingInterval': '480', 'settleCoin': 'USDT', 'copyTrading': 'none'}}]
}



#  Drakkar-Software OctoBot-Trading
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.
import async_channel.producer as channel_producer
import async_channel.util as channel_util

import octobot_commons.tentacles_management as tentacles_management

import octobot_trading.exchanges as exchanges
import octobot_trading.exchange_channel as exchange_channel


async def create_exchange_channels(exchange_manager) -> None:
    """
    Create exchange related channels
    # TODO filter creation --> not required if pause is managed
    :param exchange_manager: the related exchange manager
    """
    for exchange_channel_class_type in [exchange_channel.ExchangeChannel, exchange_channel.TimeFrameExchangeChannel]:
        await channel_util.create_all_subclasses_channel(exchange_channel_class_type, exchange_channel.set_chan,
                                                         is_synchronized=exchange_manager.is_backtesting,
                                                         exchange_manager=exchange_manager)


async def create_exchange_producers(exchange_manager, forced_producers=None) -> None:
    """
    Create exchange channels producers according to exchange manager context (backtesting, simulator, real)
    :param exchange_manager: the related exchange manager
    :param forced_producers: producers to create no anyway
    """
    import octobot_trading.personal_data as personal_data

    # Always init exchange user data first on real trading
    if _should_create_authenticated_producers(exchange_manager):
        await _create_producers(exchange_manager, personal_data.AUTHENTICATED_UPDATER_PRODUCERS)

    # Real data producers
    if _should_create_unauthenticated_producers(exchange_manager):
        import octobot_trading.exchange_data as exchange_data
        await _create_producers(exchange_manager, exchange_data.UNAUTHENTICATED_UPDATER_PRODUCERS)

    # Simulated producers
    if _should_create_simulated_producers(exchange_manager):
        await _create_producers(exchange_manager, personal_data.AUTHENTICATED_UPDATER_SIMULATOR_PRODUCERS)

    if forced_producers:
        await _create_producers(exchange_manager, forced_producers)


def _should_create_authenticated_producers(exchange_manager):
    """
    :param exchange_manager: the related exchange manager
    :return: True if should create authenticated producers
    """
    return exchange_manager.exchange.authenticated() \
           and exchange_manager.trader and exchange_manager.is_trading \
           and not (exchange_manager.is_simulated
                    or exchange_manager.is_backtesting
                    or exchange_manager.exchange_only)


def _should_create_simulated_producers(exchange_manager):
    """
    :param exchange_manager: the related exchange manager
    :return: True if should create simulated producers
    """
    return (not exchange_manager.exchange.authenticated()
            or exchange_manager.is_simulated
            or exchange_manager.is_backtesting) \
           and exchange_manager.trader and exchange_manager.is_trading and not exchange_manager.exchange_only


def _should_create_unauthenticated_producers(exchange_manager):
    """
    :param exchange_manager: the related exchange manager
    :return: True if should create unauthenticated real data producers
    """
    return not exchange_manager.is_backtesting


async def _create_producers(exchange_manager, producers_classes) -> None:
    """
    Create a list of producer instance
    :param exchange_manager: the related exchange manager
    :param producers_classes: the list of producer classes
    """
    for updater in producers_classes:
        await _create_producer(exchange_manager, updater)


async def _create_producer(exchange_manager, producer) -> channel_producer.Producer:
    """
    Create a producer instance
    :param exchange_manager: the related exchange manager
    :param producer: the producer to create
    :return: the producer instance created
    """
    should_start_producer = True
    producer_instance = producer(exchange_channel.get_chan(producer.CHANNEL_NAME, exchange_manager.id))
    if exchanges.is_channel_managed_by_websocket(exchange_manager, producer.CHANNEL_NAME):
        # websocket is handling this channel: initialize data if required
        exchange_manager.logger.debug(
            f"{exchange_manager.exchange_name} {producer.CHANNEL_NAME} channel is updated by websocket feed"
        )
        should_start_producer = \
            not exchanges.is_channel_fully_managed_by_websocket(exchange_manager, producer.CHANNEL_NAME)
        if exchanges.is_websocket_feed_requiring_init(exchange_manager, producer.CHANNEL_NAME):
            try:
                producer_instance.trigger_single_update()
            except Exception as e:
                exchange_manager.logger.exception(e, True,
                                                  f"Error when initializing data for {producer.CHANNEL_NAME} "
                                                  f"channel required by websocket: {e}")
    if should_start_producer:
        # no websocket for this channel (or channel is not fully managed by ws): start a producer
        exchange_manager.logger.debug(
            f"{exchange_manager.exchange_name} {producer.CHANNEL_NAME} channel "
            f"is updated by {producer_instance.__class__.__name__}"
        )
        await producer_instance.run()
    return producer_instance


async def create_authenticated_producer_from_parent(exchange_manager,
                                                    parent_producer_class,
                                                    force_register_producer=False):
    """
    Create an authenticated producer from its parent class
    :param exchange_manager: the related exchange manager
    :param parent_producer_class: the authenticated producer parent class
    :param force_register_producer: force the producer to register to its channel
    """
    producer = _get_authenticated_producer_from_parent(parent_producer_class)
    if producer is not None:
        producer_instance = await _create_producer(exchange_manager, producer)
        if force_register_producer:
            await producer_instance.channel.register_producer(producer_instance)


def _get_authenticated_producer_from_parent(parent_producer_class):
    """
    :param parent_producer_class: the authenticated producer parent class
    :return: the authenticated producer that inherit from parent_producer_class
    """
    import octobot_trading.personal_data as personal_data
    for authenticated_producer_candidate in personal_data.AUTHENTICATED_UPDATER_PRODUCERS:
        if tentacles_management.default_parent_inspection(authenticated_producer_candidate, parent_producer_class):
            return authenticated_producer_candidate
    return None


def requires_refresh_trigger(exchange_manager, channel):
    """
    Return True if the given channel is to be updated artificially (ex: via channel updater). In this case it
    is necessary to trigger a manual update to get the exact picture at a given time (last updater push might
    have been a few seconds ago)
    Return False if this channels updates by its exchange_manager
    and manual refresh trigger is not necessary (ex: websocket feed)
    :param exchange_manager: the related exchange manager
    :param channel: name of the channel
    :return: True if it should be refreshed via a manual trigger to be exactly up to date
    """
    return not exchanges.is_channel_managed_by_websocket(exchange_manager, channel)
